@@ -9,6 +9,6 @@ dat = imread('Pilatus.tiff')
 
 app = wx.App()
 frame = ImageFrame()
-frame.display(dat)
+frame.display(dat, contrast_level='0.005', auto_contrast=False)
 frame.Show()
 app.MainLoop()
