@@ -1,21 +1,18 @@
 #!/usr/bin/python
 #
 # wxmplott GUI to Configure Line Plots
-#
-import os
+
 from functools import partial
 import yaml
 import numpy as np
-
 import wx
 import wx.lib.colourselect  as csel
 import wx.lib.agw.flatnotebook as flat_nb
-
 import wx.lib.scrolledpanel as scrolled
-from wxutils import get_cwd
+
 from .utils import LabeledTextCtrl, MenuItem, Choice, FloatSpin
 from .config import PlotConfig
-from .colors import hexcolor, hex2rgb, mpl_color
+from .colors import hexcolor, mpl_color, GUI_COLORS
 
 FNB_STYLE = flat_nb.FNB_NO_X_BUTTON|flat_nb.FNB_SMART_TABS|flat_nb.FNB_NO_NAV_BUTTONS|flat_nb.FNB_NODRAG
 
@@ -36,7 +33,6 @@ def ffmt(val):
             pass
     return repr(val)
 
-
 def clean_texmath(txt):
     """
     clean tex math string, preserving control sequences
@@ -116,62 +112,34 @@ class PlotConfigFrame(wx.Frame):
         self.axes = self.canvas.figure.get_axes()
         self.conf.relabel()
         self.show_legend_cbs = []
+        self.wids = {}
         self.DrawPanel()
         mbar = wx.MenuBar()
-
         fmenu = wx.Menu()
         MenuItem(self, fmenu, "Save Configuration\tCtrl+S",
                  "Save Configuration",
                  self.save_config)
         MenuItem(self, fmenu, "Load Configuration\tCtrl+R",
-                 "Load Configuration",
-                 self.load_config)
+                "Load Configuration",
+                self.load_config)
         mbar.Append(fmenu, 'File')
         self.SetMenuBar(mbar)
 
-    def save_config(self, evt=None, fname='wxmplot.yaml'):
-        file_choices = 'YAML Config File (*.yaml)|*.yaml'
-        dlg = wx.FileDialog(self, message='Save plot configuration',
-                            defaultDir=get_cwd(),
-                            defaultFile=fname,
-                            wildcard=file_choices,
-                            style=wx.FD_SAVE|wx.FD_CHANGE_DIR)
-
-        if dlg.ShowModal() == wx.ID_OK:
-            conf = self.conf.get_current_config()
-            ppath = os.path.abspath(dlg.GetPath())
-            with open(ppath, 'w') as fh:
-                fh.write("%s\n" % yaml.dump(conf))
-
-
-    def load_config(self, evt=None):
-        file_choices = 'YAML Config File (*.yaml)|*.yaml'
-        dlg = wx.FileDialog(self, message='Read plot configuration',
-                            defaultDir=get_cwd(),
-                            wildcard=file_choices,
-                            style=wx.FD_OPEN)
-
-        if dlg.ShowModal() == wx.ID_OK:
-            conf = yaml.safe_load(open(os.path.abspath(dlg.GetPath()), 'r').read())
-            self.conf.load_config(conf)
-
-
     def DrawPanel(self):
         style = wx.DEFAULT_FRAME_STYLE
         wx.Frame.__init__(self, self.parent, -1, 'Configure Plot', style=style)
-        bgcol = hex2rgb('#FEFEFE')
         panel = wx.Panel(self, -1)
-        panel.SetBackgroundColour(bgcol)
 
         font = wx.Font(12, wx.SWISS, wx.NORMAL,wx.NORMAL,False)
         panel.SetFont(font)
 
         self.nb = flat_nb.FlatNotebook(panel, wx.ID_ANY, agwStyle=FNB_STYLE)
 
-        self.nb.SetActiveTabColour((253, 253, 230))
-        self.nb.SetTabAreaColour((bgcol[0]-8, bgcol[1]-8, bgcol[2]-8))
-        self.nb.SetNonActiveTabTextColour((10, 10, 100))
-        self.nb.SetActiveTabTextColour((100, 10, 10))
+        self.nb.SetTabAreaColour(GUI_COLORS.nb_area)
+        self.nb.SetActiveTabColour(GUI_COLORS.nb_active)
+        self.nb.SetNonActiveTabTextColour(GUI_COLORS.nb_text)
+        self.nb.SetActiveTabTextColour(GUI_COLORS.nb_activetext)
+
         self.nb.AddPage(self.make_linetrace_panel(parent=self.nb, font=font),
                         'Colors and Line Properties', True)
         self.nb.AddPage(self.make_range_panel(parent=self.nb, font=font),
@@ -181,9 +149,6 @@ class PlotConfigFrame(wx.Frame):
         self.nb.AddPage(self.make_scatter_panel(parent=self.nb, font=font),
                         'Scatterplot Settings',
                         self.conf.plot_type == 'scatter')
-        for i in range(self.nb.GetPageCount()):
-            self.nb.GetPage(i).SetBackgroundColour(bgcol)
-
         self.nb.SetSelection(0)
 
         sizer = wx.BoxSizer(wx.VERTICAL)
@@ -194,7 +159,58 @@ class PlotConfigFrame(wx.Frame):
         self.SetMinSize((750, 250))
         self.SetSize((1050, 475))
         self.Show()
-        self.Raise()
+
+    def save_config(self, evt=None):
+        self.conf.panel.config_save_dialog()
+
+    def load_config(self, evt=None):
+        confdict = self.conf.panel.config_load_dialog()
+
+        # check boxes
+        for attr in ('auto_margins', 'hidewith_legend',
+                     'show_grid', 'show_legend', 'show_legend_frame'):
+            self.wids[attr].SetValue(confdict[attr])
+        self.wids['show_legend_2'].SetValue(confdict['show_legend'])
+        self.wids['axes_style'].SetValue('box'==confdict['axes_style'])
+
+        # choices
+        for attr in ('theme', 'legend_loc', 'legend_onaxis', 'zoom_style'):
+            self.wids[attr].SetStringSelection(confdict[attr])
+
+        xscale = confdict['xscale']
+        yscale = confdict['yscale']
+        self.wids['logchoice'].SetStringSelection(f"x {xscale} / y {yscale}")
+
+        # colors
+        for attr in ('facecolor', 'textcolor', 'framecolor', 'gridcolor',
+                     'scatter_normalcolor', 'scatter_normaledge',
+                     'scatter_selectcolor', 'scatter_selectedge'):
+            self.wids[attr].SetColour(confdict[attr])
+
+        # floats
+        for attr in ('labelfont', 'legendfont', 'titlefont',
+                     'viewpad', 'scatter_size'):
+            self.wids[attr].SetValue(confdict[attr])
+
+        margin_vals = confdict['margins']
+        for i in range(4):
+            self.wids['margins'][i].SetValue(margin_vals[i])
+
+        for i, trace in enumerate(confdict['traces']):
+            w = self.wids[f'trace_{i}']
+            w['color'].SetColour(trace['color'])
+            w['style'].SetStringSelection(trace['style'])
+            w['marker'].SetStringSelection(trace['marker'])
+            w['drawstyle'].SetStringSelection(trace['drawstyle'])
+            w['linewidth'].SetValue(trace['linewidth'])
+            w['alpha'].SetValue(trace['alpha'])
+            w['markersize'].SetValue(trace['markersize'])
+            w['zorder'].SetValue(trace['zorder'])
+            w['fill'].SetValue(trace['fill'])
+            if i == 0:
+                self.wids['def_thickness'].SetValue(trace['linewidth'])
+                self.wids['def_markersize'].SetValue(trace['markersize'])
+
 
     def make_range_panel(self, parent, font=None):
         # bounds, margins, scales
@@ -209,6 +225,7 @@ class PlotConfigFrame(wx.Frame):
         logchoice = wx.Choice(panel, choices=self.conf.log_choices,  size=(200,-1))
         logchoice.SetStringSelection("x %s / y %s" % (self.conf.xscale, self.conf.yscale))
         logchoice.Bind(wx.EVT_CHOICE, self.onLogScale)
+        self.wids['logchoice'] = logchoice
 
         sizer.Add(mtitle,     (1, 0), (1,1), labstyle, 2)
         sizer.Add(logchoice,  (1, 1), (1,3), labstyle, 2)
@@ -217,6 +234,8 @@ class PlotConfigFrame(wx.Frame):
         # Zoom
         ztitle = wx.StaticText(panel, -1, 'Zoom Style: ')
         zoomchoice = wx.Choice(panel, choices=self.conf.zoom_choices, size=(200,-1))
+        self.wids['zoom_style'] = zoomchoice
+
         if self.conf.zoom_style in self.conf.zoom_choices:
             zoomchoice.SetStringSelection(self.conf.zoom_style)
         zoomchoice.Bind(wx.EVT_CHOICE, self.onZoomStyle)
@@ -239,21 +258,52 @@ class PlotConfigFrame(wx.Frame):
             auto_b.SetValue(self.conf.user_limits[laxes] == 4*[None])
         except:
             pass
-
+        
+        self.vpad_val = FloatSpin(panel, value=2.5, min_val=0, max_val=100,
+                                  increment=0.5, digits=2,
+                                  size=(FSPINSIZE, -1), action=self.onViewPadEvent)
+        self.wids['viewpad'] = self.vpad_val
+       
         xb0, xb1 = laxes.get_xlim()
         yb0, yb1 = laxes.get_ylim()
-        if user_lims[0] is not None: xb0 = user_lims[0]
-        if user_lims[1] is not None: xb1 = user_lims[1]
+        if user_lims[0] is not None:
+            xb0 = user_lims[0]
+        if user_lims[1] is not None:
+            xb1 = user_lims[1]
 
-        if user_lims[2] is not None: yb0 = user_lims[2]
-        if user_lims[3] is not None: yb1 = user_lims[3]
+        if user_lims[2] is not None:
+            yb0 = user_lims[2]
+        if user_lims[3] is not None:
+            yb1 = user_lims[3]
 
         y2b0, y2b1 = [None, None]
         if raxes is not None:
             y2b0, y2b1 = raxes.get_ylim()
             user_lims = self.conf.user_limits[raxes]
-            if user_lims[2] is not None: y2b0 = user_lims[2]
-            if user_lims[3] is not None: y2b1 = user_lims[3]
+            if user_lims[2] is not None:
+                y2b0 = user_lims[2]
+            if user_lims[3] is not None:
+                y2b1 = user_lims[3]
+
+        y3b0, y3b1 = [None, None]
+        if len(axes) > 2:
+            ax3 = axes[2]
+            y3b0, y3b1 = ax3.get_ylim()
+            user_lims = self.conf.user_limits[ax3]
+            if user_lims[2] is not None:
+                y3b0 = user_lims[2]
+            if user_lims[3] is not None:
+                y3b1 = user_lims[3]
+
+        y4b0, y4b1 = [None, None]
+        if len(axes) > 3:
+            ax4 = axes[3]
+            y4b0, y4b1 = ax4.get_ylim()
+            user_lims = self.conf.user_limits[ax4]
+            if user_lims[2] is not None:
+                y4b0 = user_lims[2]
+            if user_lims[3] is not None:
+                    y4b1 = user_lims[3]
 
         opts = dict(size=(125, -1), labeltext='', action=self.onBounds)
 
@@ -263,24 +313,37 @@ class PlotConfigFrame(wx.Frame):
                          LabeledTextCtrl(panel,value=ffmt(yb1), **opts)]
         self.y2bounds = [LabeledTextCtrl(panel,value=ffmt(y2b0), **opts),
                          LabeledTextCtrl(panel,value=ffmt(y2b1), **opts)]
+        self.y3bounds = [LabeledTextCtrl(panel,value=ffmt(y3b0), **opts),
+                         LabeledTextCtrl(panel,value=ffmt(y3b1), **opts)]
+        self.y4bounds = [LabeledTextCtrl(panel,value=ffmt(y4b0), **opts),
+                         LabeledTextCtrl(panel,value=ffmt(y4b1), **opts)]
 
-        self.vpad_val = FloatSpin(panel, value=2.5, min_val=0, max_val=100,
-                                  increment=0.5, digits=2,
-                                  size=(FSPINSIZE, -1),
-                                  action=self.onViewPadEvent)
+
+        self.y3offset = FloatSpin(panel, value=self.conf.y3offset, min_val=0,
+                                  max_val=0.8, increment=0.01, digits=3,
+                                  size=(FSPINSIZE, -1), action=self.onY3Offset)
 
         if user_lims == 4*[None]:
             [w.Disable() for w in self.xbounds]
             [w.Disable() for w in self.ybounds]
-
-        if raxes is None:
             [w.Disable() for w in self.y2bounds]
+            [w.Disable() for w in self.y3bounds]
+            [w.Disable() for w in self.y4bounds]
+        else:
+            for yb in self.y4bounds:
+                yb.Enable(len(axes) > 3)
+            for yb in self.y3bounds:
+                yb.Enable(len(axes) > 2)
+            for yb in self.y2bounds:
+                yb.Enable(len(axes) > 1)
 
         btext  = 'Plot Boundaries : '
         ptext  = ' Padding (% of Data Range): '
         xtext  = '   X axis:'
         ytext  = '   Y axis:'
         y2text = '   Y2 axis:'
+        y3text = '   Y3 axis:'
+        y4text = '   Y4 axis:'
         def showtext(t):
             return wx.StaticText(panel, -1, t)
 
@@ -304,6 +367,17 @@ class PlotConfigFrame(wx.Frame):
         sizer.Add(showtext(' : '),  (6, 2), (1, 1), labstyle, 2)
         sizer.Add(self.y2bounds[1], (6, 3), (1, 1), labstyle, 2)
 
+        sizer.Add(showtext(y3text), (7, 0), (1, 1), labstyle, 2)
+        sizer.Add(self.y3bounds[0], (7, 1), (1, 1), labstyle, 2)
+        sizer.Add(showtext(' : '),  (7, 2), (1, 1), labstyle, 2)
+        sizer.Add(self.y3bounds[1], (7, 3), (1, 1), labstyle, 2)
+
+        sizer.Add(showtext(y4text), (8, 0), (1, 1), labstyle, 2)
+        sizer.Add(self.y4bounds[0], (8, 1), (1, 1), labstyle, 2)
+        sizer.Add(showtext(' : '),  (8, 2), (1, 1), labstyle, 2)
+        sizer.Add(self.y4bounds[1], (8, 3), (1, 1), labstyle, 2)
+
+        irow = 9
         # Margins
         _left, _top, _right, _bot = ["%.3f"% x for x in self.conf.margins]
 
@@ -327,15 +401,22 @@ class PlotConfigFrame(wx.Frame):
         auto_m  = wx.CheckBox(panel,-1, ' Default ', (-1, -1), (-1, -1))
         auto_m.Bind(wx.EVT_CHECKBOX,self.onAutoMargin)
         auto_m.SetValue(self.conf.auto_margins)
+        self.wids['auto_margins'] = auto_m
+        self.wids['margins'] = (lmarg, rmarg, bmarg, tmarg)
 
         msizer = wx.BoxSizer(wx.HORIZONTAL)
         msizer.AddMany((ltitle, lmarg, rtitle, rmarg,
                         btitle, bmarg, ttitle, tmarg))
 
-        sizer.Add(mtitle,  (8, 0), (1,1), labstyle, 2)
-        sizer.Add(auto_m,  (8, 1), (1,1), labstyle, 2)
-        sizer.Add(msizer,  (9, 1), (1,6), labstyle, 2)
+        sizer.Add(mtitle,  (irow, 0), (1,1), labstyle, 2)
+        sizer.Add(auto_m,  (irow, 1), (1,1), labstyle, 2)
+        sizer.Add(msizer,  (irow+1, 1), (1,6), labstyle, 2)
 
+        irow += 2
+        y3title = wx.StaticText(panel, -1, 'Extra Padding Y3 & Y4 Axes: ')
+        self.y3offset
+        sizer.Add(y3title,  (irow, 0), (1,2), labstyle, 2)
+        sizer.Add(self.y3offset,  (irow, 2), (1,3), labstyle, 2)
         autopack(panel, sizer)
         return panel
 
@@ -373,6 +454,13 @@ class PlotConfigFrame(wx.Frame):
                                   mpl_color(conf.scatter_selectedge,
                                            default=(200, 0, 0)),
                                   size=(25, 25))
+
+        self.wids['scatter_size'] = ssize
+        self.wids['scatter_normalcolor'] = nfcol
+        self.wids['scatter_normaledge'] = necol
+        self.wids['scatter_selectcolor'] = sfcol
+        self.wids['scatter_selectedge'] = secol
+
         nfcol.Bind(csel.EVT_COLOURSELECT, partial(self.onScatter, item='scatt_nf'))
         necol.Bind(csel.EVT_COLOURSELECT, partial(self.onScatter, item='scatt_ne'))
         sfcol.Bind(csel.EVT_COLOURSELECT, partial(self.onScatter, item='scatt_sf'))
@@ -417,24 +505,50 @@ class PlotConfigFrame(wx.Frame):
         self.titl = LabeledTextCtrl(panel, self.conf.title.replace('\n', '\\n'),
                                     action = partial(self.onText, item='title'),
                                     labeltext='Title: ', size=(475, -1))
+        self.xlab = LabeledTextCtrl(panel, self.conf.xlabel.replace('\n', '\\n'),
+                                    action = partial(self.onText, item='xlabel'),
+                                    labeltext='X Label: ', size=(475, -1))
         self.ylab = LabeledTextCtrl(panel, self.conf.ylabel.replace('\n', '\\n'),
                                     action = partial(self.onText, item='ylabel'),
                                     labeltext='Y Label: ', size=(475, -1))
         self.y2lab= LabeledTextCtrl(panel, self.conf.y2label.replace('\n', '\\n'),
                                     action = partial(self.onText, item='y2label'),
                                     labeltext='Y2 Label: ', size=(475, -1))
-        self.xlab = LabeledTextCtrl(panel, self.conf.xlabel.replace('\n', '\\n'),
-                                    action = partial(self.onText, item='xlabel'),
-                                    labeltext='X Label: ', size=(475, -1))
+        self.y3lab= LabeledTextCtrl(panel, self.conf.y3label.replace('\n', '\\n'),
+                                    action = partial(self.onText, item='y3label'),
+                                    labeltext='Y3 Label: ', size=(475, -1))
+        self.y4lab= LabeledTextCtrl(panel, self.conf.y4label.replace('\n', '\\n'),
+                                    action = partial(self.onText, item='y4label'),
+                                    labeltext='Y4 Label: ', size=(475, -1))
+
+        self.yax_color = wx.CheckBox(panel,-1, 'Use Trace Color for Y Axes', (-1, -1), (-1, -1))
+        self.yax_color.Bind(wx.EVT_CHECKBOX, self.onYaxes_tracecolor)
+        self.yax_color.SetValue(self.conf.yaxes_tracecolor)
+
+        axes = self.canvas.figure.get_axes()
+        self.yax_color.Enable(len(axes) > 1)
+        self.y2lab.label.Enable(len(axes) > 1)
+        self.y3lab.label.Enable(len(axes) > 2)
+        self.y4lab.label.Enable(len(axes) > 3)
+        self.y2lab.Enable(len(axes) > 1)
+        self.y3lab.Enable(len(axes) > 2)
+        self.y4lab.Enable(len(axes) > 3)
 
         sizer.Add(self.titl.label,  (0, 0), (1, 1), labstyle)
-        sizer.Add(self.titl,        (0, 1), (1, 4), labstyle)
-        sizer.Add(self.ylab.label,  (1, 0), (1, 1), labstyle)
-        sizer.Add(self.ylab,        (1, 1), (1, 4), labstyle)
-        sizer.Add(self.y2lab.label, (2, 0), (1, 1), labstyle)
-        sizer.Add(self.y2lab,       (2, 1), (1, 4), labstyle)
-        sizer.Add(self.xlab.label,  (3, 0), (1, 1), labstyle)
-        sizer.Add(self.xlab,        (3, 1), (1, 4), labstyle)
+        sizer.Add(self.titl,        (0, 1), (1, 6), labstyle)
+        sizer.Add(self.xlab.label,  (1, 0), (1, 1), labstyle)
+        sizer.Add(self.xlab,        (1, 1), (1, 6), labstyle)
+        sizer.Add(self.ylab.label,  (2, 0), (1, 1), labstyle)
+        sizer.Add(self.ylab,        (2, 1), (1, 6), labstyle)
+        sizer.Add(self.y2lab.label, (3, 0), (1, 1), labstyle)
+        sizer.Add(self.y2lab,       (3, 1), (1, 6), labstyle)
+        sizer.Add(self.y3lab.label, (4, 0), (1, 1), labstyle)
+        sizer.Add(self.y3lab,       (4, 1), (1, 6), labstyle)
+        sizer.Add(self.y4lab.label, (5, 0), (1, 1), labstyle)
+        sizer.Add(self.y4lab,       (5, 1), (1, 6), labstyle)
+        sizer.Add(self.yax_color,   (6, 0), (1, 6), labstyle)
+
+        irow = 7
 
         t0 = wx.StaticText(panel, -1, 'Text Sizes:', style=labstyle)
         t1 = wx.StaticText(panel, -1, 'Titles:', style=labstyle)
@@ -445,32 +559,51 @@ class PlotConfigFrame(wx.Frame):
         ttl_size = FloatSpin(panel, value=self.conf.labelfont.get_size(),
                              action=partial(self.onText, item='titlesize'),
                              **fsopts)
-
-        leg_size = FloatSpin(panel, value=self.conf.legendfont.get_size(),
-                             action=partial(self.onText, item='legendsize'),
-                             **fsopts)
         lab_size = FloatSpin(panel, value=self.conf.labelfont.get_size(),
                              action=partial(self.onText, item='labelsize'),
                              **fsopts)
+        leg_size = FloatSpin(panel, value=self.conf.legendfont.get_size(),
+                             action=partial(self.onText, item='legendsize'),
+                             **fsopts)
 
         self.title_fontsize = ttl_size
         self.legend_fontsize = leg_size
         self.label_fontsize = lab_size
+        self.wids['titlefont'] = ttl_size
+        self.wids['labelfont'] = lab_size
+        self.wids['legendfont'] = leg_size
 
-        sizer.Add(t0,        (4, 0), (1, 1), labstyle)
-        sizer.Add(t1,        (4, 1), (1, 1), labstyle)
-        sizer.Add(ttl_size,  (4, 2), (1, 2), labstyle)
-        sizer.Add(t2,        (5, 1), (1, 1), labstyle)
-        sizer.Add(lab_size,  (5, 2), (1, 2), labstyle)
-        sizer.Add(t3,        (6, 1), (1, 1), labstyle)
-        sizer.Add(leg_size,  (6, 2), (1, 2), labstyle)
 
+        sizer.Add(t0,        (irow, 0), (1, 1), labstyle)
+        sizer.Add(t1,        (irow, 1), (1, 1), labstyle)
+        sizer.Add(ttl_size,  (irow, 2), (1, 1), labstyle)
+        sizer.Add(t2,        (irow, 3), (1, 1), labstyle)
+        sizer.Add(lab_size,  (irow, 4), (1, 1), labstyle)
+        sizer.Add(t3,        (irow, 5), (1, 1), labstyle)
+        sizer.Add(leg_size,  (irow, 6), (1, 1), labstyle)
+
+        irow = irow + 1
         # Legend
-        bstyle=wx.ALIGN_LEFT|wx.ALIGN_CENTER_VERTICAL|wx.ST_NO_AUTORESIZE
-        labstyle= wx.ALIGN_LEFT|wx.ALIGN_CENTER_VERTICAL
+        # bstyle = wx.ALIGN_LEFT|wx.ALIGN_CENTER_VERTICAL|wx.ST_NO_AUTORESIZE
+        labstyle = wx.ALIGN_LEFT|wx.ALIGN_CENTER_VERTICAL
 
-        ax = self.axes[0]
+        # ax = self.axes[0]
+        show_leg = wx.CheckBox(panel,-1, 'Show Legend', (-1, -1), (-1, -1))
+        show_leg.Bind(wx.EVT_CHECKBOX,partial(self.onShowLegend, item='legend'))
+        show_leg.SetValue(self.conf.show_legend)
+        if show_leg not in self.show_legend_cbs:
+            self.show_legend_cbs.append(show_leg)
 
+        show_lfr = wx.CheckBox(panel,-1, 'Show Legend Frame', (-1, -1), (-1, -1))
+        show_lfr.Bind(wx.EVT_CHECKBOX,partial(self.onShowLegend,item='frame'))
+        show_lfr.SetValue(self.conf.show_legend_frame)
+
+        togg_leg  = wx.CheckBox(panel,-1, 'Click Legend to Show/Hide Line',
+                                (-1, -1), (-1, -1))
+        togg_leg.Bind(wx.EVT_CHECKBOX, self.onHideWithLegend)
+        togg_leg.SetValue(self.conf.hidewith_legend)
+
+        
         leg_ttl = wx.StaticText(panel, -1, 'Legend:', size=(-1, -1), style=labstyle)
         loc_ttl = wx.StaticText(panel, -1, 'Location:', size=(-1, -1), style=labstyle)
         leg_loc = wx.Choice(panel, -1, choices=self.conf.legend_locs, size=(150, -1))
@@ -482,35 +615,27 @@ class PlotConfigFrame(wx.Frame):
         leg_onax.Bind(wx.EVT_CHOICE,partial(self.onShowLegend, item='onaxis'))
         leg_onax.SetStringSelection(self.conf.legend_onaxis)
 
-        togg_leg  = wx.CheckBox(panel,-1, 'Click Legend to Show/Hide Line',
-                                (-1, -1), (-1, -1))
-        togg_leg.Bind(wx.EVT_CHECKBOX, self.onHideWithLegend)
-        togg_leg.SetValue(self.conf.hidewith_legend)
-
-        show_leg = wx.CheckBox(panel,-1, 'Show Legend', (-1, -1), (-1, -1))
-        show_leg.Bind(wx.EVT_CHECKBOX,partial(self.onShowLegend, item='legend'))
-        show_leg.SetValue(self.conf.show_legend)
-        if show_leg not in self.show_legend_cbs:
-            self.show_legend_cbs.append(show_leg)
-
-        show_lfr = wx.CheckBox(panel,-1, 'Show Legend Frame', (-1, -1), (-1, -1))
-        show_lfr.Bind(wx.EVT_CHECKBOX,partial(self.onShowLegend,item='frame'))
-        show_lfr.SetValue(self.conf.show_legend_frame)
+        self.wids['hidewith_legend'] = togg_leg
+        self.wids['legend_loc'] = leg_loc
+        self.wids['legend_onaxis'] = leg_onax
+        self.wids['show_legend_2'] = show_leg
+        self.wids['show_legend_frame'] = show_lfr
 
         lsizer = wx.BoxSizer(wx.HORIZONTAL)
 
         lsizer.AddMany((leg_ttl, show_leg, show_lfr, togg_leg))
-        sizer.Add(lsizer,    (8, 0), (1, 8), labstyle, 2)
+        sizer.Add(lsizer,    (irow, 0), (1, 8), labstyle, 2)
+
+        irow += 1
 
         lsizer = wx.BoxSizer(wx.HORIZONTAL)
         lsizer.AddMany((loc_ttl, leg_loc, leg_onax))
-        sizer.Add(lsizer,  (9, 1), (1, 4), labstyle, 2)
+        sizer.Add(lsizer,  (irow, 1), (1, 5), labstyle, 2)
         autopack(panel, sizer)
         return panel
 
     def make_linetrace_panel(self, parent, font=None):
         """colours and line properties"""
-
         panel = scrolled.ScrolledPanel(parent, size=(900, 250),
                                        style=wx.GROW|wx.TAB_TRAVERSAL, name='p1')
         if font is None:
@@ -520,13 +645,12 @@ class PlotConfigFrame(wx.Frame):
         i = 0
         cnf = self.conf
         ax = self.axes[0]
-
         labstyle= wx.ALIGN_LEFT|wx.ALIGN_CENTER_VERTICAL
 
         theme_names = list(cnf.themes.keys())
         themechoice = Choice(panel, choices=theme_names, action=self.onTheme)
         themechoice.SetStringSelection(cnf.current_theme)
-
+        self.wids['theme'] = themechoice
 
         textcol = csel.ColourSelect(panel, label=" Text ", size=(80, -1),
                                     colour=mpl_color(cnf.textcolor))
@@ -537,8 +661,8 @@ class PlotConfigFrame(wx.Frame):
         fbgcol = csel.ColourSelect(panel,  label=" Frame ", size=(80, -1),
                                    colour=mpl_color(self.canvas.figure.get_facecolor()))
 
-        self.colwids = {'text': textcol, 'face': bgcol,
-                        'grid': gridcol, 'frame': fbgcol}
+        self.wids.update({'textcolor': textcol, 'facecolor': bgcol,
+                           'gridcolor': gridcol, 'framecolor': fbgcol})
 
         bgcol.Bind(csel.EVT_COLOURSELECT,   partial(self.onColor, item='face'))
         fbgcol.Bind(csel.EVT_COLOURSELECT,  partial(self.onColor, item='frame'))
@@ -548,16 +672,20 @@ class PlotConfigFrame(wx.Frame):
         show_grid  = wx.CheckBox(panel,-1, ' Show Grid  ')
         show_grid.Bind(wx.EVT_CHECKBOX,self.onShowGrid)
         show_grid.SetValue(cnf.show_grid)
-
-        show_box  = wx.CheckBox(panel,-1, ' Show Top/Right Axes  ')
-        show_box.Bind(wx.EVT_CHECKBOX, self.onShowBox)
-        show_box.SetValue(cnf.axes_style == 'box')
-
+        self.wids['show_grid'] = show_grid
+        
         show_leg = wx.CheckBox(panel,-1, 'Show Legend  ')
         show_leg.Bind(wx.EVT_CHECKBOX,partial(self.onShowLegend, item='legend'))
         show_leg.SetValue(cnf.show_legend)
         if show_leg not in self.show_legend_cbs:
             self.show_legend_cbs.append(show_leg)
+        self.wids['show_legend'] = show_leg
+        
+        show_box  = wx.CheckBox(panel,-1, ' Show Top/Right Axes  ')
+        show_box.Bind(wx.EVT_CHECKBOX, self.onShowBox)
+        show_box.SetValue(cnf.axes_style == 'box')
+        self.wids['axes_style'] = show_box
+
 
         tsizer = wx.BoxSizer(wx.HORIZONTAL)
         tsizer.Add(wx.StaticText(panel, -1, ' Theme: '), 0, labstyle, 3)
@@ -592,11 +720,12 @@ class PlotConfigFrame(wx.Frame):
         tsizer.Add(wx.StaticText(panel, -1, ' Symbol Size: '), 0, labstyle, 3)
         tsizer.Add(msz, 0, labstyle, 3)
 
+        self.wids['def_thickness'] = allthk
+        self.wids['def_markersize'] = msz
 
         sizer.Add(tsizer,    (3, 0), (1, 9), labstyle, 3)
 
         irow = 4
-
         for t in ('#','Label','Color', 'Alpha', 'Fill', 'Line Style',
                   'Thickness', 'Symbol',
                   'Size', 'Z Order', 'Join Style'):
@@ -607,10 +736,10 @@ class PlotConfigFrame(wx.Frame):
         self.trace_labels = []
         self.choice_linewidths = []
         self.choice_markersizes = []
-        ntrace_display = min(cnf.ntrace+3, len(cnf.traces))
+        ntrace_display = min(cnf.ntrace+2, len(cnf.traces))
         for i in range(ntrace_display):
             irow += 1
-            label  = "trace %i" % i
+            # label  = "trace %i" % i
             lin  = cnf.traces[i]
             dlab = lin.label
             dcol = hexcolor(lin.color)
@@ -629,42 +758,46 @@ class PlotConfigFrame(wx.Frame):
             col = csel.ColourSelect(panel,  -1, "", dcol, size=(25, 25))
             col.Bind(csel.EVT_COLOURSELECT, partial(self.onColor, trace=i))
 
-            self.colwids[i] = col
+            alp = FloatSpin(panel, size=(FSPINSIZE, -1), value=dalp,
+                            min_val=0, max_val=1, increment=0.05, digits=2,
+                            action=partial(self.onAlpha, trace=i))
+
+            ffil = wx.CheckBox(panel, -1, '')
+            ffil.Bind(wx.EVT_CHECKBOX, partial(self.onFill, trace=i))
+            ffil.SetValue(dfill)
+
+            sty = Choice(panel, choices=cnf.styles, size=(150,-1),
+                         action=partial(self.onStyle,trace=i))
+            sty.SetStringSelection(dsty)
 
             thk = FloatSpin(panel, size=(ISPINSIZE, -1), value=dthk,
                             min_val=0, max_val=20, increment=0.5, digits=1,
                             action=partial(self.onThickness, trace=i))
             self.choice_linewidths.append(thk)
 
-            sty = Choice(panel, choices=cnf.styles, size=(150,-1),
-                         action=partial(self.onStyle,trace=i))
-            sty.SetStringSelection(dsty)
-
-            msz = FloatSpin(panel, size=(FSPINSIZE, -1), value=dmsz,
-                            min_val=0, max_val=30, increment=0.5, digits=1,
-                            action=partial(self.onMarkerSize, trace=i))
-            self.choice_markersizes.append(msz)
-            zor = FloatSpin(panel, size=(ISPINSIZE, -1), value=dzord,
-                            min_val=-500, max_val=500, increment=1, digits=0,
-                            action=partial(self.onZorder, trace=i))
-
             sym = Choice(panel, choices=cnf.symbols, size=(125,-1),
                          action=partial(self.onSymbol,trace=i))
 
             sym.SetStringSelection(dsym)
+            
+            msz = FloatSpin(panel, size=(FSPINSIZE, -1), value=dmsz,
+                            min_val=0, max_val=30, increment=0.5, digits=1,
+                            action=partial(self.onMarkerSize, trace=i))
+            self.choice_markersizes.append(msz)
+            
+            zor = FloatSpin(panel, size=(ISPINSIZE, -1), value=dzord,
+                            min_val=-500, max_val=500, increment=1, digits=0,
+                            action=partial(self.onZorder, trace=i))
+
 
             jsty = wx.Choice(panel, -1, choices=cnf.drawstyles, size=(125,-1))
             jsty.Bind(wx.EVT_CHOICE, partial(self.onJoinStyle, trace=i))
             jsty.SetStringSelection(djsty)
 
-            ffil = wx.CheckBox(panel, -1, '')
-            ffil.Bind(wx.EVT_CHECKBOX, partial(self.onFill, trace=i))
-            ffil.SetValue(dfill)
-
-            alp = FloatSpin(panel, size=(FSPINSIZE, -1), value=dalp,
-                            min_val=0, max_val=1, increment=0.05, digits=2,
-                            action=partial(self.onAlpha, trace=i))
-
+            self.wids[f'trace_{i}'] = {'color': col, 'alpha': alp, 'fill': ffil,
+                                       'style': sty, 'linewidth': thk,
+                                       'marker': sym, 'markersize': msz,
+                                       'zorder': zor, 'drawstyle': jsty}
 
             sizer.Add(lab.label,(irow,0),(1,1),wx.ALIGN_LEFT|wx.ALL, 4)
             sizer.Add(lab, (irow,1),(1,1),wx.ALIGN_LEFT|wx.ALL, 4)
@@ -678,11 +811,13 @@ class PlotConfigFrame(wx.Frame):
             sizer.Add(zor, (irow,9),(1,1),wx.ALIGN_LEFT|wx.ALL, 4)
             sizer.Add(jsty, (irow,10),(1,1),wx.ALIGN_LEFT|wx.ALL, 4)
 
-
         autopack(panel,sizer)
-        panel.SetupScrolling()
+        wx.CallAfter(panel.SetupScrolling)
         return panel
 
+    def onYaxes_tracecolor(self, event=None):
+        self.conf.set_yaxes_tracecolor(yaxes_tracecolor=event.IsChecked(), delay_draw=False)
+
     def onResetLines(self, event=None):
         pass
 
@@ -693,7 +828,9 @@ class PlotConfigFrame(wx.Frame):
 
         if item == 'trace':
             self.conf.set_trace_color(color, trace=trace)
-            self.colwids[trace].SetColour(color)
+            self.wids[f'trace_{trace}']['color'].SetColour(color)
+            if self.conf.show_legend:
+                self.conf.draw_legend()
 
         elif item == 'grid':
             self.conf.set_gridcolor(color)
@@ -711,10 +848,10 @@ class PlotConfigFrame(wx.Frame):
         theme = event.GetString()
         conf = self.conf
         conf.set_theme(theme=theme)
-        self.colwids['text'].SetColour(conf.textcolor)
-        self.colwids['grid'].SetColour(conf.gridcolor)
-        self.colwids['face'].SetColour(conf.facecolor)
-        self.colwids['frame'].SetColour(conf.framecolor)
+        self.wids['textcolor'].SetColour(conf.textcolor)
+        self.wids['gridcolor'].SetColour(conf.gridcolor)
+        self.wids['facecolor'].SetColour(conf.facecolor)
+        self.wids['framecolor'].SetColour(conf.framecolor)
 
         self.title_fontsize.SetValue(self.conf.titlefont.get_size())
         self.legend_fontsize.SetValue(self.conf.legendfont.get_size())
@@ -723,9 +860,9 @@ class PlotConfigFrame(wx.Frame):
         for i in range(ntrace_display):
             try:
                 lin = self.conf.traces[i]
-                curcol = hexcolor(self.colwids[i].GetColour())
+                curcol = hexcolor(self.wids[f'trace_{i}']['color'].GetColour())
                 newcol = hexcolor(lin.color)
-                self.colwids[i].SetColour(newcol)
+                self.wids[f'trace_{i}']['color'].SetColour(newcol)
                 if newcol != curcol:
                     self.conf.set_trace_color(newcol, trace=i)
             except KeyError:
@@ -780,7 +917,7 @@ class PlotConfigFrame(wx.Frame):
             self.conf.set_trace_linewidth(val, trace=trace)
 
 
-    def onAutoBounds(self,event):
+    def onAutoBounds(self, event):
         axes = self.canvas.figure.get_axes()
         if event.IsChecked():
             for ax in axes:
@@ -788,6 +925,8 @@ class PlotConfigFrame(wx.Frame):
             [m.Disable() for m in self.xbounds]
             [m.Disable() for m in self.ybounds]
             [m.Disable() for m in self.y2bounds]
+            [m.Disable() for m in self.y3bounds]
+            [m.Disable() for m in self.y4bounds]
             self.vpad_val.Enable()
             self.conf.unzoom(full=True)
         else:
@@ -805,6 +944,17 @@ class PlotConfigFrame(wx.Frame):
                 for m, v in zip(self.y2bounds, y2b):
                     m.Enable()
                     m.SetValue(ffmt(v))
+            if len(axes) > 2:
+                y3b = axes[2].get_ylim()
+                for m, v in zip(self.y3bounds, y3b):
+                    m.Enable()
+                    m.SetValue(ffmt(v))
+            if len(axes) > 3:
+                y4b = axes[3].get_ylim()
+                for m, v in zip(self.y4bounds, y4b):
+                    m.Enable()
+                    m.SetValue(ffmt(v))
+
 
     def onBounds(self, event=None):
         def FloatNone(v):
@@ -823,10 +973,15 @@ class PlotConfigFrame(wx.Frame):
         if len(axes) > 1:
             y2min, y2max = [FloatNone(w.GetValue()) for w in self.y2bounds]
             self.conf.user_limits[axes[1]] = [xmin, xmax, y2min, y2max]
+        if len(axes) > 2:
+            y3min, y3max = [FloatNone(w.GetValue()) for w in self.y3bounds]
+            self.conf.user_limits[axes[2]] = [xmin, xmax, y3min, y3max]
+        if len(axes) > 3:
+            y4min, y4max = [FloatNone(w.GetValue()) for w in self.y4bounds]
+            self.conf.user_limits[axes[3]] = [xmin, xmax, y4min, y4max]
         self.conf.set_viewlimits()
         self.conf.canvas.draw()
 
-
     def onAutoMargin(self,event):
         self.conf.auto_margins = event.IsChecked()
         if self.conf.auto_margins:
@@ -843,6 +998,10 @@ class PlotConfigFrame(wx.Frame):
         left, top, right, bottom = [float(w.GetValue()) for w in self.margins]
         self.conf.set_margins(left=left, top=top, right=right, bottom=bottom)
 
+    def onY3Offset(self, event=None):
+        self.conf.y3offset = float(self.y3offset.GetValue())
+        self.conf.relabel()
+
     def onViewPadEvent(self, event=None):
 
         self.conf.viewpad = float(self.vpad_val.GetValue())
@@ -916,6 +1075,10 @@ class PlotConfigFrame(wx.Frame):
             wid = self.ylab
         elif item == 'y2label':
             wid = self.y2lab
+        elif item == 'y3label':
+            wid = self.y3lab
+        elif item == 'y4label':
+            wid = self.y4lab
         elif item == 'xlabel':
             wid = self.xlab
         elif item == 'trace':
@@ -927,13 +1090,15 @@ class PlotConfigFrame(wx.Frame):
         except TypeError:
             s = ''
 
-        if item in ('xlabel', 'ylabel', 'y2label', 'title'):
+        if item in ('xlabel', 'ylabel', 'y2label', 'y3label', 'y4label', 'title'):
             try:
                 kws = {item: s}
                 self.conf.relabel(**kws)
-                wid.SetBackgroundColour((255, 255, 255))
-            except: # as from latex error!
-                wid.SetBackgroundColour((250, 250, 150))
+                wid.SetBackgroundColour(GUI_COLORS.text_bg)
+                wid.SetForegroundColour(GUI_COLORS.text)
+            except: # as from latex error!a
+                wid.SetBackgroundColour(GUI_COLORS.text_invalid_bg)
+                wid.SetForegroundColour(GUI_COLORS.text_invalid)
         elif item == 'trace':
             try:
                 self.conf.set_trace_label(s, trace=trace)
@@ -945,7 +1110,8 @@ class PlotConfigFrame(wx.Frame):
 
     def onShowBox(self, event=None):
         style='box'
-        if not event.IsChecked(): style='open'
+        if not event.IsChecked():
+            style='open'
         self.conf.set_axes_style(style=style)
 
     def onShowLegend(self, event, item=''):
