@@ -2,31 +2,47 @@
 """
 wxmplot PlotPanel: a wx.Panel for line plotting, using matplotlib
 """
-import os
-import sys
 from functools import partial
 from datetime import datetime
-
+from pathlib import Path
+import yaml
 import wx
 
 from numpy import nonzero, where
 import matplotlib as mpl
-from matplotlib.dates import date2num, datestr2num
+from matplotlib.dates import date2num, datestr2num, num2date
+from matplotlib.dates import AutoDateLocator
 
 from matplotlib.figure import Figure
 from matplotlib.ticker import FuncFormatter
 from matplotlib.backends.backend_wxagg import FigureCanvasWxAgg as FigureCanvas
 from matplotlib.gridspec import GridSpec
 from matplotlib.colors import colorConverter
-from matplotlib.collections import CircleCollection
 
+from wxutils import get_cwd
 from .basepanel import BasePanel
-from .config import PlotConfig, ifnot_none
-from .utils import inside_poly, fix_filename, gformat, MenuItem
+from .config import PlotConfig, ifnot_none, SIDE_YAXES
+from .utils import inside_poly, MenuItem, fix_filename
 from .plotconfigframe import PlotConfigFrame
 
 to_rgba = colorConverter.to_rgba
 
+def format_date(x, xrange, tz=None):
+    if xrange > 12: # 12 days
+        dtformat = "%Y-%m-%d"
+    elif xrange > 3: # 3 days
+        dtformat = "%Y-%m-%d %H"
+    elif xrange > 0.25: # 6 hours
+        dtformat = "%m-%d %H:%M"
+    elif xrange > 0.12: #
+        dtformat = "%d %H:%M:%S"
+    elif xrange > 0.01: # 4.4 seconds
+        dtformat = "%H:%M:%S.%f"
+    else:
+        dtformat = "%M:%S.%f"
+    return datetime.strftime(num2date(x, tz=tz), dtformat)
+
+
 class PlotPanel(BasePanel):
     """
     MatPlotlib line plot as a wx.Panel, suitable for embedding
@@ -71,12 +87,11 @@ class PlotPanel(BasePanel):
         self.use_dates = False
         self.dates_style = None
 
-    def plot(self, xdata, ydata, side='left', title=None,
-             xlabel=None, ylabel=None, y2label=None,
-             use_dates=False, dates_style=None, **kws):
+    def plot(self, xdata, ydata, title=None, xlabel=None, ylabel=None,
+             y2label=None, y3label=None, y4label=None, use_dates=False,
+             dates_style=None, yaxes=1, side=None, **kws):
         """
         create a new plot of x/y data, clearing any existing plot on the panel
-
         """
         allaxes = self.fig.get_axes()
         if len(allaxes) > 1:
@@ -89,11 +104,14 @@ class PlotPanel(BasePanel):
         self.conf.zoom_lims = []
         self.conf.axes_traces = {}
         self.clear()
-        axes = self.axes
-        if side == 'right':
-            axes = self.get_right_axes()
+        yaxes, axes = self.get_yaxes(yaxes, side=side)
+
         self.conf.reset_lines()
         self.conf.yscale = 'linear'
+        self.conf.yscale = 'linear'
+        self.conf.y2scale = 'linear'
+        self.conf.y3scale = 'linear'
+        self.conf.y4scale = 'linear'
         self.conf.user_limits[axes] = 4*[None]
 
         if xlabel is not None:
@@ -102,41 +120,51 @@ class PlotPanel(BasePanel):
             self.set_ylabel(ylabel, delay_draw=True)
         if y2label is not None:
             self.set_y2label(y2label, delay_draw=True)
+        if y3label is not None:
+            self.set_y3label(y3label, delay_draw=True)
+        if y4label is not None:
+            self.set_y4label(y4label, delay_draw=True)
         if title is not None:
             self.set_title(title, delay_draw=True)
         self.dates_style = ifnot_none(dates_style, self.dates_style)
         self.use_dates = ifnot_none(use_dates, self.use_dates)
-        return self.oplot(xdata, ydata, side=side, **kws)
+        return self.oplot(xdata, ydata, yaxes=yaxes, **kws)
 
 
-    def oplot(self, xdata, ydata, side='left', label=None, xlabel=None,
-              ylabel=None, y2label=None, title=None, dy=None,
-              ylog_scale=None, xlog_scale=None, grid=None, xmin=None,
-              xmax=None, ymin=None, ymax=None, color=None, style=None, alpha=None,
-              fill=False, drawstyle=None, linewidth=2,
-              marker=None, markersize=None, refresh=True, show_legend=None,
+    def oplot(self, xdata, ydata, label=None, xlabel=None, ylabel=None,
+              y2label=None, y3label=None, y4label=None, title=None, dy=None,
+              xlog_scale=None, ylog_scale=None, y2log_scale=None, y3log_scale=None,
+              y4log_scale=None, grid=None, xmin=None,
+              xmax=None, ymin=None, ymax=None, color=None, style=None,
+              alpha=None, fill=False, drawstyle=None, linewidth=2, marker=None,
+              markersize=None, refresh=True, show_legend=None,
               legend_loc='best', legend_on=True, delay_draw=False,
               bgcolor=None, framecolor=None, gridcolor=None, textcolor=None,
               labelfontsize=None, titlefontsize=None, legendfontsize=None,
               fullbox=None, axes_style=None, zorder=None, viewpad=None,
-              theme=None, use_dates=None, dates_style=None, **kws):
+              theme=None, use_dates=None, dates_style=None, timezone=None,
+              yaxes=1, side=None, yaxes_tracecolor=None, **kws):
+
         """
         basic plot method, adding to an existing display
-
         """
-
         self.cursor_mode = 'zoom'
         conf = self.conf
         conf.plot_type = 'lineplot'
-        axes = self.axes
 
         if theme is not None:
             conf.set_theme(theme=theme)
-        if side == 'right':
-            axes = self.get_right_axes()
+
+        yaxes, axes = self.get_yaxes(yaxes, side=side)
         # set y scale to log/linear
         if ylog_scale is not None:
             conf.yscale = {False:'linear', True:'log'}[ylog_scale]
+        if y2log_scale is not None:
+            conf.y2scale = {False:'linear', True:'log'}[y2log_scale]
+        if y3log_scale is not None:
+            conf.y3scale = {False:'linear', True:'log'}[y3log_scale]
+        if y4log_scale is not None:
+            conf.y3scale = {False:'linear', True:'log'}[y4log_scale]
 
         if xlog_scale is not None:
             conf.xscale = {False:'linear', True:'log'}[xlog_scale]
@@ -145,6 +173,7 @@ class PlotPanel(BasePanel):
 
         self.dates_style = ifnot_none(dates_style, self.dates_style)
         self.use_dates = ifnot_none(use_dates, self.use_dates)
+
         if isinstance(xdata[0], datetime):
             self.use_dates = True
 
@@ -153,14 +182,18 @@ class PlotPanel(BasePanel):
             #   1. xdate are in datetime: convert to mpl dates
             #   2. xdata are strings: parse with datestr2num
             #   3. xdata are floats: convert as unix timestamp to mpl dates
+            axes.xaxis.set_major_locator(AutoDateLocator())
             x0 = xdata[0]
             dstyle = self.dates_style
             if dstyle is None:
                 dstyle = ''
             if isinstance(x0, datetime):
+                self.dates_tzinfo = xdata[0].tzinfo
                 xdata = date2num(xdata)
             elif isinstance(x0, str) or dstyle.lower().startswith('str'):
                 xdata = datestr2num(xdata)
+            if timezone is not None:
+                self.dates_tzinfo = timezone
 
         linewidth = ifnot_none(linewidth, 2)
         conf.viewpad = ifnot_none(viewpad, conf.viewpad)
@@ -171,6 +204,10 @@ class PlotPanel(BasePanel):
             self.set_ylabel(ylabel, delay_draw=True)
         if y2label is not None:
             self.set_y2label(y2label, delay_draw=True)
+        if y3label is not None:
+            self.set_y3label(y3label, delay_draw=True)
+        if y4label is not None:
+            self.set_y4label(y4label, delay_draw=True)
         if title  is not None:
             self.set_title(title, delay_draw=True)
         if show_legend is not None:
@@ -189,10 +226,12 @@ class PlotPanel(BasePanel):
         conf.user_limits[axes][2] = ifnot_none(ymin, conf.user_limits[axes][2])
         conf.user_limits[axes][3] = ifnot_none(ymax, conf.user_limits[axes][3])
 
-        if axes == self.axes:
-            axes.yaxis.set_major_formatter(FuncFormatter(self.yformatter))
-        else:
-            axes.yaxis.set_major_formatter(FuncFormatter(self.y2formatter))
+        yformatter = {1: self.yformatter,
+                      2: self.y2formatter,
+                      3: self.y3formatter,
+                      4: self.y4formatter}.get(yaxes, self.yformatter)
+
+        axes.yaxis.set_major_formatter(FuncFormatter(yformatter))
 
         zorder = ifnot_none(zorder, 5*(conf.ntrace+1))
 
@@ -210,6 +249,7 @@ class PlotPanel(BasePanel):
             conf.set_framecolor(framecolor)
 
         conf.set_trace_zorder(zorder, delay_draw=True)
+        conf.set_trace_yaxes(yaxes, delay_draw=True)
         if color:
             conf.set_trace_color(color, delay_draw=True)
         if style:
@@ -262,9 +302,14 @@ class PlotPanel(BasePanel):
         else:
             axes.grid(False)
 
-        if (self.conf.xscale == 'log' or self.conf.yscale == 'log'):
+        if (self.conf.xscale == 'log' or self.conf.yscale == 'log'
+            or self.conf.y2scale == 'log' or self.conf.y3scale == 'log'
+            or self.conf.y4scale == 'log'):
             self.set_logscale(xscale=self.conf.xscale,
                               yscale=self.conf.yscale,
+                              y2scale=self.conf.y2scale,
+                              y3scale=self.conf.y3scale,
+                              y4scale=self.conf.y4scale,
                               delay_draw=True)
 
         if label is None:
@@ -311,14 +356,17 @@ class PlotPanel(BasePanel):
             conf.axes_style = axes_style
 
         conf.set_axes_style(delay_draw=delay_draw)
+        conf.set_yaxes_tracecolor(yaxes_tracecolor, delay_draw=True)
+
         if not delay_draw:
             self.draw()
             # self.canvas.Refresh()
         conf.ntrace = conf.ntrace + 1
         return _lines
 
-    def plot_many(self, datalist, side='left', title=None,
-                  xlabel=None, ylabel=None, show_legend=False, zoom_limits=None, **kws):
+    def plot_many(self, datalist, title=None, xlabel=None, ylabel=None,
+                  show_legend=False, zoom_limits=None, yaxes=1, side=None,
+                  **kws):
         """
         plot many traces at once, taking a list of (x, y) pairs
         """
@@ -337,8 +385,8 @@ class PlotPanel(BasePanel):
 
 
         conf = self.conf
-        opts = dict(side=side, title=title, xlabel=xlabel, ylabel=ylabel,
-                    delay_draw=True, show_legend=False)
+        opts = {'yaxes': yaxes, 'title': title, 'xlabel': xlabel,
+               'ylabel': ylabel, 'delay_draw': True, 'show_legend': False}
         opts.update(kws)
         x0, y0, opts = unpack_tracedata(datalist[0], **opts)
 
@@ -355,23 +403,21 @@ class PlotPanel(BasePanel):
             x, y, opts = unpack_tracedata(dat, delay_draw=True)
             self.oplot(x, y, **opts)
 
-        self.reset_formats()
         self.set_zoomlimits(zoom_limits)
         self.conf.show_legend = show_legend
         if show_legend:
             conf.draw_legend(delay_draw=True)
         conf.relabel(delay_draw=True)
+        self.reset_formats()
         self.draw()
         # self.canvas.Refresh()
 
-
     def get_zoomlimits(self):
         return self.axes, self.get_viewlimits(), self.conf.zoom_lims
 
     def set_zoomlimits(self, limits):
         """set zoom limits returned from get_zoomlimits()"""
         if limits is None:
-            # print("panel.set_zoom none")
             return False
         ax, vlims, zoom_lims = limits
         if ax == self.axes:
@@ -385,39 +431,111 @@ class PlotPanel(BasePanel):
                 return False
         return True
 
-
-    def add_text(self, text, x, y, side='left', size=None,
-                 rotation=None, ha='left', va='center',
-                 family=None, **kws):
+    def add_text(self, text, x, y, size=None, rotation=None, ha='left',
+                 va='center', yaxes=1, side=None, **kws):
         """add text at supplied x, y position
         """
-        axes = self.axes
-        if side == 'right':
-            axes = self.get_right_axes()
-        dynamic_size = False
-        if size is None:
-            size = self.conf.legendfont.get_size()
-            dynamic_size = True
+        yaxes, axes = self.get_yaxes(yaxes, side=side)
+        size = self.conf.legendfont.get_size()
         t = axes.text(x, y, text, ha=ha, va=va, size=size,
-                      rotation=rotation, family=family, **kws)
-        self.conf.added_texts.append((dynamic_size, t))
+                      rotation=rotation, **kws)
+        self.conf.added_texts.append(t)
         self.draw()
 
-    def add_arrow(self, x1, y1, x2, y2,  side='left',
-                  shape='full', color='black',
-                  width=0.01, head_width=0.03, overhang=0, **kws):
+    def add_arrow(self, x1, y1, x2, y2, shape='full', color='black',
+                  width=0.01, head_width=0.03, overhang=0, yaxes=1, side=None,
+                  delay_draw=False, **kws):
         """add arrow supplied x, y position"""
         dx, dy = x2-x1, y2-y1
 
-        axes = self.axes
-        if side == 'right':
-            axes = self.get_right_axes()
+        yaxes, axes = self.get_yaxes(yaxes, side=side)
         axes.arrow(x1, y1, dx, dy, shape=shape,
                    length_includes_head=True,
                    fc=color, edgecolor=color,
                    width=width, head_width=head_width,
                    overhang=overhang, **kws)
-        self.draw()
+        if not delay_draw:
+            self.draw()
+
+    def add_vline(self, x, ymin=0, ymax=1, side=None, yaxes=1,
+                      delay_draw=False, label='_nolegend_', report_data=None, **kws):
+        """add a vertical line at x
+
+        Args:
+           x (float):      x position of line, in user coordinates
+           ymin (float):   starting y fraction (window units -- not user units!)
+           ymax (float):   ending y y fraction (window units -- not user units!)
+           delay_draw (bool): whether to delay drawing
+           label (str):    label to show in legend
+           report_data (dict or None): data to report on mouse cursor (left-down)
+        """
+        yaxes, axes = self.get_yaxes(yaxes, side=side)
+        vline = axes.axvline(x, ymin=ymin, ymax=ymax, label=label, **kws)
+        if report_data is not None:
+            self.conf.marker_report_data.append((x, None, label, report_data))
+        if not delay_draw:
+            self.draw()
+        return vline
+
+    def add_hline(self, y, xmin=0, xmax=1, side=None, yaxes=1,
+                      delay_draw=False, label='_nolegend_', report_data=None, **kws):
+        """add a horizontal line at y
+
+        Args:
+           y (float):      y position of line, in user coordinates
+           xmin (float):   starting x fraction (window units -- not user units!)
+           xmax (float):   ending y x fraction (window units -- not user units!)
+           delay_draw (bool): whether to delay drawing
+           label (str):    label to show in legend
+           report_data (dict or None): data to report on mouse cursor (left-down)
+        """
+        yaxes, axes = self.get_yaxes(yaxes, side=side)
+        hline = axes.axhline(y, xmin=xmin, xmax=xmax, label=label, **kws)
+        if report_data is not None:
+            self.conf.marker_report_data.append((None, y, label, report_data))
+        if not delay_draw:
+            self.draw()
+        return hline
+
+    def add_marker(self, x, y, marker='o', size=4, color='black', side=None, yaxes=1,
+                      delay_draw=False, label='_nolegend_', report_data=None, **kws):
+        """add a marker at x, y coordinates
+
+        Args:
+           x (float):      x position of line, in user coordinates
+           y (float):      y position of line, in user coordinates
+           marker (str):   marker type
+           size (float):   marker size
+           color (str):   marker color
+           delay_draw (bool): whether to delay drawing
+           label (str):    label to show in legend
+           report_data (dict or None): data to report on mouse cursor (left-down)
+        """
+        yaxes, axes = self.get_yaxes(yaxes, side=side)
+        mline = self.oplot([x], [y], yaxes=yaxes, marker=marker, markersizer=size,
+                           color=color, label=label, *kws)
+        if report_data is not None:
+            self.conf.marker_report_data.append((x, y, label, report_data))
+        if not delay_draw:
+            self.draw()
+        return mline
+
+    def set_xtick_labels(self, xticks, yaxes=1, side=None):
+        """
+        set xtick labels from dict of {x: label} pairs
+        """
+        yaxes, axes = self.get_yaxes(yaxes, side=side)
+        axes.set_xticks(list(xticks.keys()))
+        axes.set_xticklabels(list(xticks.values()))
+
+    def set_ytick_labels(self, yticks, yaxes=1, side=None):
+        """
+        set ytick labels from dict of {y: label} pairs
+        """
+        yaxes, axes = self.get_yaxes(yaxes, side=side)
+        axes.set_yticks(list(yticks.keys()))
+        axes.set_yticklabels(list(yticks.values()))
+
 
     def scatterplot(self, xdata, ydata, label=None, size=10,
                     color=None, edgecolor=None,
@@ -523,12 +641,11 @@ class PlotPanel(BasePanel):
             self.lasso_callback(data = sdat,
                                 selected=pts, mask=mask)
 
-    def set_xylims(self, limits, axes=None, side='left'):
+    def set_xylims(self, limits, axes=None, yaxes=1, side=None):
         "set user-defined limits and apply them"
         if axes is None:
-            axes = self.axes
-            if side == 'right':
-                axes = self.get_right_axes()
+            yaxes, axes = self.get_yaxes(yaxes, side=side)
+
         self.conf.user_limits[axes] = list(limits)
         self.unzoom_all()
 
@@ -541,11 +658,44 @@ class PlotPanel(BasePanel):
         self.conf.set_viewlimits()
 
     def get_viewlimits(self, axes=None):
-        if axes is None: axes = self.axes
+        if axes is None:
+                axes = self.axes
         xmin, xmax = axes.get_xlim()
         ymin, ymax = axes.get_ylim()
         return (xmin, xmax, ymin, ymax)
 
+    def get_yaxes(self, n, side=None):
+        """get y axis number 1, 2, 3, or 4, where
+        n=1 is the normal left-hand y-axis (aka "y"),
+        n=2 is the right-hand y-axis (aka "y2"), and
+        n=3 is a second right-hand y-axis (aka "y3"), and
+        n=4 is a third right-hand y-axis (aka "y4")
+        """
+        if side is not None:
+            _n = SIDE_YAXES.get(side, None)
+            if _n in (1, 2, 3, 4):
+                n = _n
+        if n not in (1, 2, 3, 4):
+            raise ValueError("get_yaxes() needs value 1, 2, 3, or 4")
+
+        while len(self.fig.get_axes()) < n:
+            self.axes.twinx()
+        return (n, self.fig.get_axes()[n-1])
+
+    def get_right_axes(self, side='right'):
+        """
+        return right-hand (y2, y3, or y4) axes, creating if needed)
+
+        use side='right' or 'y2' [default], or side='right2', or 'y3', or
+       'right3' or 'y4',   See also  get_yaxes()
+        """
+        if side in ('right3', 'y4'):
+            return self.get_yaxes(4)[1]
+        elif side in ('right2', 'y3'):
+            return self.get_yaxes(3)[1]
+        else:
+            return self.get_yaxes(2)[1]
+
     def clear(self):
         """ clear plot """
         for ax in self.fig.get_axes():
@@ -554,6 +704,8 @@ class PlotPanel(BasePanel):
         self.conf.xlabel = ''
         self.conf.ylabel = ''
         self.conf.y2label = ''
+        self.conf.y3label = ''
+        self.conf.y4label = ''
         self.conf.title  = ''
         self.conf.data_save = {}
 
@@ -561,6 +713,7 @@ class PlotPanel(BasePanel):
         """reset configuration to defaults."""
         self.conf.set_defaults()
 
+
     def unzoom(self, event=None, **kws):
         """ zoom out 1 level, or to full data range """
         self.reset_formats()
@@ -596,10 +749,15 @@ class PlotPanel(BasePanel):
 
             self.conf.process_data()
 
-    def set_logscale(self, event=None, xscale='linear', yscale='linear',
+    def set_logscale(self, event=None, xscale='linear',
+                     yscale='linear', y2scale='linear',
+                     y3scale='linear', y4scale='linear',
                      delay_draw=False):
         "set log or linear scale for x, y axis"
         self.conf.set_logscale(xscale=xscale, yscale=yscale,
+                               y2scale=y2scale,
+                               y3scale=y3scale,
+                               y4scale=y4scale,
                                delay_draw=delay_draw)
 
     def toggle_legend(self, evt=None, show=None):
@@ -615,6 +773,14 @@ class PlotPanel(BasePanel):
             show = not self.conf.show_grid
         self.conf.enable_grid(show)
 
+    def set_config(self, **conf):
+        """set configuration with dict of configuration options"""
+        self.conf.set_config(**conf)
+
+    def get_config(self, **conf):
+        """get configuration as dict of configuration options"""
+        return self.conf.get_config()
+
     def configure(self, event=None):
         """show configuration frame"""
         if self.win_config is not None:
@@ -629,6 +795,39 @@ class PlotPanel(BasePanel):
                                               trace_color_callback=self.trace_color_callback)
             self.win_config.Raise()
 
+    def config_save_dialog(self, evt=None):
+        title = self.conf.title.strip()
+        if len(title) < 2:
+            title = 'wxmplot'
+        fname = fix_filename(f'{title}.yaml')
+
+        file_choices = 'YAML Config File (*.yaml)|*.yaml'
+        dlg = wx.FileDialog(self, message='Save plot configuration',
+                            defaultDir=get_cwd(),
+                            defaultFile=fname,
+                            wildcard=file_choices,
+                            style=wx.FD_SAVE|wx.FD_CHANGE_DIR)
+
+        if dlg.ShowModal() == wx.ID_OK:
+            conf = self.get_config()
+            text = yaml.dump(conf, default_flow_style=None, indent=5, sort_keys=False)
+            ppath = Path(dlg.GetPath())
+            with open(ppath, 'w', encoding='utf-8') as fh:
+                 fh.write(f"{text}\n")
+
+
+    def config_load_dialog(self, evt=None):
+        file_choices = 'YAML Config File (*.yaml)|*.yaml'
+        dlg = wx.FileDialog(self, message='Read plot configuration',
+                            defaultDir=get_cwd(),
+                            wildcard=file_choices,
+                            style=wx.FD_OPEN)
+        conf = None
+        if dlg.ShowModal() == wx.ID_OK:
+            conf = yaml.safe_load(open(Path(dlg.GetPath()), 'r').read())
+            self.set_config(**conf)
+        return conf
+
     ####
     ## create GUI
     ####x
@@ -708,24 +907,23 @@ class PlotPanel(BasePanel):
         trans = self.fig.transFigure.inverted().transform
 
         # Static margins
-        l, t, r, b = self.axesmargins
-        (l, b), (r, t) = trans(((l, b), (r, t)))
+        left, top, right, bot = self.axesmargins
+        (left, bot), (right, top) = trans(((left, bot), (right, top)))
 
         # Extent
         dl, dt, dr, db = 0, 0, 0, 0
-        for ax in self.fig.get_axes():
+        for i, ax in enumerate(self.fig.get_axes()):
             (x0, y0),(x1, y1) = ax.get_position().get_points()
             try:
                 (ox0, oy0), (ox1, oy1) = ax.get_tightbbox(self.canvas.get_renderer()).get_points()
                 (ox0, oy0), (ox1, oy1) = trans(((ox0 ,oy0),(ox1 ,oy1)))
                 dl = min(0.2, max(dl, (x0 - ox0)))
                 dt = min(0.2, max(dt, (oy1 - y1)))
-                dr = min(0.2, max(dr, (ox1 - x1)))
+                dr = min(0.2*(i+1), max(dr, (ox1 - x1)))
                 db = min(0.2, max(db, (y0 - oy0)))
-            except:
+            except ValueError:
                 pass
-
-        return (l + dl, t + dt, r + dr, b + db)
+        return [float(left+dl), float(top+dt), float(right+dr), float(bot+db)]
 
     def autoset_margins(self):
         """auto-set margins  left, bottom, right, top
@@ -736,26 +934,26 @@ class PlotPanel(BasePanel):
         if not self.conf.auto_margins:
             return
 
-        self.conf.margins = l, t, r, b = self.get_default_margins()
-        self.gridspec.update(left=l, top=1-t, right=1-r, bottom=b)
+        self.conf.margins = left, top, right, bot = self.get_default_margins()
+        self.gridspec.update(left=left, top=1-top, right=1-right, bottom=bot)
         # Axes positions update
         for ax in self.fig.get_axes():
-            figpos = ax.get_subplotspec().get_position(self.canvas.figure)
-            ax.set_position(figpos)
+            try:
+                figpos = ax.get_subplotspec().get_position(self.canvas.figure)
+                ax.set_position(figpos)
+            except Exception:
+                pass
 
     def draw(self):
         self.canvas.draw()
 
-    def update_line(self, trace, xdata, ydata, side='left', draw=False,
-                    update_limits=True):
+    def update_line(self, trace, xdata, ydata,draw=False,
+                    update_limits=True, yaxes=1, side=None):
         """ update a single trace, for faster redraw """
         x = self.conf.get_mpl_line(trace)
         if trace >= self.conf.ntrace:
-            self.oplot(xdata, ydata, side=side, delay_draw=True)
+            self.oplot(xdata, ydata, yaxes=yaxes, side=side, delay_draw=True)
         x.set_data(xdata, ydata)
-        axes = self.axes
-        if side == 'right':
-            axes = self.get_right_axes()
 
         if update_limits:
             self.set_viewlimits()
@@ -800,18 +998,65 @@ class PlotPanel(BasePanel):
         except:
             x, y = event.xdata, event.ydata
 
+        ax  = self.canvas.figure.get_axes()[0]
         if x is not None and y is not None:
-            msg = "X,Y= %g, %g" % (x, y)
+            if self.use_dates:
+                xlims = ax.get_xlim()
+                xrange = abs(xlims[1] - xlims[0])
+                x = format_date(x, xrange, tz=self.dates_tzinfo)
+            else:
+                x = f"{x:g}"
+            msg = f"X,Y= {x}, {y:g}"
         if len(self.fig.get_axes()) > 1:
             ax2 = self.fig.get_axes()[1]
             try:
                 x2, y2 = ax2.transData.inverted().transform((ex, ey))
-                msg = "X,Y,Y2= %g, %g, %g" % (x, y, y2)
+                msg = f"X,Y,Y2= {x}, {y:g}, {y2:g}"
             except:
                 pass
 
         nsbar = getattr(self, 'nstatusbar', 1)
-        self.write_message(msg,  panel=max(0, nsbar - 2))
+        self.write_message(msg,  panel=max(0, nsbar-2))
         if (self.cursor_callback is not None and
             hasattr(self.cursor_callback , '__call__')):
-            self.cursor_callback(x=event.xdata, y=event.ydata)
+            marker_data = []
+            if (len(self.conf.marker_report_data) > 0 and
+                    event.xdata is not None and
+                    event.ydata is not None):
+                xlims = ax.get_xlim()
+                ylims = ax.get_ylim()
+                xrange = max(xlims[1] - xlims[0], 1.e-14)
+                yrange = max(ylims[1] - ylims[0], 1.e-14)
+                for mx, my, label, extra in self.conf.marker_report_data:
+                    nearx, neary = True, True
+                    if mx is not None:
+                        nearx = (abs(event.xdata-mx) /xrange)  < 0.01
+                    if my is not None:
+                        neary = (abs(event.ydata-my) /yrange)  < 0.01
+                    if nearx and neary:
+                        marker_data.append((mx, my, label, extra))
+
+            self.cursor_callback(x=event.xdata, y=event.ydata,
+                                 message=msg, marker_data=marker_data)
+
+    def report_motion(self, event=None):
+        if event.inaxes is None:
+            return
+        x, y  = event.xdata, event.ydata
+        if len(self.fig.get_axes()) > 1:
+            try:
+                x, y = self.axes.transData.inverted().transform((event.x, event.y))
+            except:
+                pass
+        if x is not None and y is not None:
+            if self.use_dates:
+                ax  = self.canvas.figure.get_axes()[0]
+                xlims = ax.get_xlim()
+                xrange = abs(xlims[1] - xlims[0])
+                x = format_date(x, xrange, tz=self.dates_tzinfo)
+            else:
+                x = f"{x:g}"
+            msg = f"X,Y= {x}, {y:g}"
+
+        nsbar = getattr(self, 'nstatusbar', 1)
+        self.write_message(msg, panel=max(0, nsbar-1))
